package com.malk.guyuan.service.tencent.impl;

import com.alibaba.fastjson.JSON;
import com.malk.guyuan.server.tencent.TXYConf;
import com.malk.guyuan.service.tencent.TXYInvoice;
import com.malk.server.common.McException;
import com.malk.utils.UtilMap;
import com.malk.utils.UtilNumber;
import com.tencentcloudapi.common.Credential;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import com.tencentcloudapi.common.profile.ClientProfile;
import com.tencentcloudapi.common.profile.HttpProfile;
import com.tencentcloudapi.ocr.v20181119.OcrClient;
import com.tencentcloudapi.ocr.v20181119.models.MixedInvoiceOCRRequest;
import com.tencentcloudapi.ocr.v20181119.models.MixedInvoiceOCRResponse;
import com.tencentcloudapi.ocr.v20181119.models.VatInvoiceVerifyNewRequest;
import com.tencentcloudapi.ocr.v20181119.models.VatInvoiceVerifyNewResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;

@Slf4j
@Service
public class TXYImplInvoice implements TXYInvoice {

    @Autowired
    private TXYConf txyConf;

    // 创建请求描述
    private ClientProfile doRequest(String endPoint) {

        HttpProfile httpProfile = new HttpProfile();
        httpProfile.setEndpoint(endPoint);
        ClientProfile clientProfile = new ClientProfile();
        clientProfile.setHttpProfile(httpProfile);
        return clientProfile;
    }

    /**
     * 混贴票据识别
     *
     * @apiNote https://cloud.tencent.com/document/product/866/37835
     */
    @Override
    public Map doMixedInvoiceOCR(String imageUrl) throws TencentCloudSDKException {

        Credential cred = new Credential(txyConf.getSecretId(), txyConf.getSecretKey());
        ClientProfile clientProfile = doRequest("ocr.tencentcloudapi.com");
        OcrClient client = new OcrClient(cred, txyConf.getRegion(), clientProfile);

        MixedInvoiceOCRRequest req = new MixedInvoiceOCRRequest();
        req.setImageUrl(imageUrl);
        req.setReturnMultiplePage(true); // PDF多页识别, 仅支持返回文件前30页
        MixedInvoiceOCRResponse resp = client.MixedInvoiceOCR(req);
        String result = MixedInvoiceOCRResponse.toJsonString(resp);
        log.debug("请求响应, {}", result);
        return (Map) JSON.parse(result);
    }

    /**
     * 发票验真[新版]
     *
     * @apiNote https://cloud.tencent.com/document/product/866/73674
     */
    @Override
    public Map VatInvoiceVerifyNew(String invoiceKind, String invoiceCode, String invoiceNo, String invoiceDate, String amount, String checkCode, String excludingTax) throws TencentCloudSDKException {
        Credential cred = new Credential(txyConf.getSecretId(), txyConf.getSecretKey());
        ClientProfile clientProfile = doRequest("ocr.tencentcloudapi.com");
        OcrClient client = new OcrClient(cred, txyConf.getRegion(), clientProfile);

        if (StringUtils.isNotBlank(checkCode)) {
            checkCode = checkCode.substring(checkCode.length() - 6);
        }

        VatInvoiceVerifyNewRequest req = new VatInvoiceVerifyNewRequest();
        req.setInvoiceNo(invoiceNo);
        req.setInvoiceDate(invoiceDate);
        req.setInvoiceCode(invoiceCode);
        req.setCheckCode(checkCode);
        if (invoiceKind.contains("增值税电子") || invoiceKind.contains("电子发票") || invoiceKind.contains("全电发票")) {
            // 全电票, 需要价税合计且无发票代码与校验码
            req.setCheckCode(null);
            req.setInvoiceCode(null);
            req.setAmount(amount);
        } else {
            req.setAmount(null);
            req.setAmount(excludingTax);
        }
        VatInvoiceVerifyNewResponse resp = client.VatInvoiceVerifyNew(req);
        String result = VatInvoiceVerifyNewResponse.toJsonString(resp);
        Map rsp = (Map) JSON.parse(result);

        // 因全电票取值, 取值价税合计, 单独校验下金额
        Map invoice = (Map) rsp.get("Invoice");
        McException.assertAccessException(!UtilNumber.equalBigDecimal(UtilMap.getString(invoice, "AmountWithTax"), amount), "发票有疑问, 价税合计金额不匹配!");
        McException.assertAccessException(!UtilNumber.equalBigDecimal(UtilMap.getString(invoice, "AmountWithoutTax"), excludingTax), "发票有疑问, 不含税金额不匹配!");

        log.debug("请求响应, {}", result);
        return rsp;
    }
}
