package com.malk.service.aliwork.impl;

import com.alibaba.fastjson.JSON;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.aliwork.YDClient;
import com.malk.service.aliwork.YDService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.*;

@Slf4j
@Service
public class YDServiceImpl implements YDService {

    @Autowired
    private YDClient ydClient;

    /**
     * 操作数据 [异步] - 审批通过立即更新, 会无效
     */
    @Async
    @SneakyThrows
    @Override
    public Object operateData(YDParam ydParam, YDConf.FORM_OPERATION type) {
        Thread.sleep(3000);
        return ydClient.operateData(ydParam, type);
    }


    /**
     * 子表全部数据获取, 最大分页为50 [可考虑中间表思路]
     */
    @Override
    public List<Map> queryDetails(YDParam ydParam) {
        return _queryDetails(ydParam, new ArrayList<>());
    }

    // 递归查询
    private List<Map> _queryDetails(YDParam ydParam, List<Map> details) {
        ydParam.setPageSize(50);
        DDR_New ddr_new = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_details);
        details.addAll((List<Map>) ddr_new.getData());
        if (ddr_new.getTotalCount() != details.size()) {
            ydParam.setPageNumber(ydParam.getPageNumber() + 1);
            _queryDetails(ydParam, details);
        }
        return details;
    }

    /**
     * 全表复制
     */
    @Override
    public Object copyFormData(YDParam cParam, YDParam sParam) {

        Map<String, Object> formData = ydClient.queryData(cParam, YDConf.FORM_QUERY.retrieve_id).getFormData();

        Map newFormData = new HashMap();
        for (String compId : formData.keySet()) {
            if (compId.endsWith("_id") || compId.endsWith("_value")) {
                continue;
            }
            newFormData.put(compId, _formatCompData(compId, formData));
        }

        newFormData.put("employeeField_lia74rki", Arrays.asList(YDConf.PUB_ACCOUNT));

        sParam.setFormDataJson(JSON.toJSONString(newFormData));
        ydClient.operateData(sParam, YDConf.FORM_OPERATION.start);

        return null;
    }

    /// 格式化组件
    Object _formatCompData(String compId, Map<String, Object> formData) {

        // todo: 成员, 附件, 关联表单 ignoreDetails 忽略明细
//        List<Map> association = JSON.parseArray(String.valueOf(JSON.parse(String.valueOf(formData.get("associationFormField_lg0gmbi8_id")))), Map.class);
//        String formInstId = String.valueOf(association.get(0).get("instanceId"));


        if (compId.startsWith("tableField_")) {
            List<Map> details = (List<Map>) formData.get(compId);
            for (Map<String, Object> detail : details) {
                for (String compId_sub : detail.keySet()) {
                    detail.put(compId_sub, _formatCompData(compId_sub, detail));
                }
            }
        }
        if (compId.startsWith("employeeField_")) {
            return formData.get(compId + "_id");
        }
        if (compId.startsWith("associationFormField_")) {
            return JSON.parse(String.valueOf(formData.get(compId + "_id")));
        }

        return formData.get(compId);
    }
}
