package com.malk.service.dingtalk;

import java.util.List;
import java.util.Map;

/**
 * 钉钉通讯录管理
 *
 * @apiNote https://open.dingtalk.com/document/orgapp/contacts-overview
 */
public interface DDClient_Contacts {

    /**
     * 获取子部门ID列表 [获取企业部门下的所有直属子部门列表]
     */
    List<Number> listSubDepartmentId(String access_token, Number dept_id);

    /**
     * 获取全部架构内部门_全部 ID
     */
    List<Number> getDepartmentId_all(String access_token);

    /**
     * 获取部门列表
     */
    List<Map> listSubDepartmentDetail(String access_token, Number dept_id);

    /**
     * 获取全部架构内部门_全部 detail
     */
    List<Map> getDepartmentDetail_all(String access_token);

    /**
     * 获取部门用户userid列表 [无需分页]
     */
    List<String> listDepartmentUserId(String access_token, Number dept_id);

    /**
     * 查询用户详情
     */
    Map getUserInfoById(String access_token, String userId);

    /**
     * 根据手机号查询用户
     */
    Map getUserInfoByMobile(String access_token, String mobile);

    /**
     * 删除用户
     */
    boolean deleteUser(String access_token, String userId);

    /**
     * 创建用户 [24小时只能收到一次邀请通知, 若是频繁退出再进入, 可关闭邀请确认后自动加入, 路径: 设置 - 安全中心 - 隐私开关 - 团队添加我时需要我的确认 - 关闭]
     */
    Map createUser(String access_token, String name, String mobile, List<String> dept_id_list, Map body_ext);

    /**
     * 创建部门
     */
    Map createDepartment(String access_token, String name, long parent_id, Map body_ext);


    /**
     * 获取员工人数
     *
     * @param only_active 是否包含未激活钉钉人数
     */
    int getUserCount(String access_token, boolean only_active);

}
