package com.malk.service.dingtalk.impl;

import cn.hutool.core.util.ObjectUtil;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.dingtalk.DDR;
import com.malk.service.dingtalk.DDClient_Attendance;
import com.malk.utils.UtilHttp;
import com.malk.utils.UtilMap;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 考勤
 *
 * @apiNote https://open.dingtalk.com/document/orgapp/attendance-overview
 */
@Service
@Slf4j
public class DDImplClient_Attendance implements DDClient_Attendance {

    /**
     * 获取打卡结果
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/open-attendance-clock-in-data
     */
    @Override
    public List<Map> listAttendanceResult(String access_token, String[] userIdList, String workDateFrom, String workDateTo, Number offset, Number limit) {
        Map param = UtilMap.map("access_token", access_token);
        Map body = UtilMap.map("userIdList, workDateFrom, workDateTo, offset, limit, isI18n", userIdList, workDateFrom, workDateTo, offset, limit, false);
        return DDR.doPost("https://oapi.dingtalk.com/attendance/list", null, param, body).getRecordresult();
    }

    /**
     * 获取打卡详情
     *
     * @param userIds       企业内的员工ID列表，最大值50;
     * @param checkDateFrom checkDateFrom 和workDateTo参数相隔最多7天（包含7天）; 注: 参数传"2021-12-01 18:00:00"，员工在19:00的打卡信息获取不到
     */
    @Override
    public List<Map> listAttendanceRecord(String access_token, List userIds, String checkDateFrom, String checkDateTo) {
        Map param = UtilMap.map("access_token", access_token);
        Map body = UtilMap.map("userIds, checkDateFrom, checkDateTo, isI18n", userIds, checkDateFrom, checkDateTo, false);
        return DDR.doPost("https://oapi.dingtalk.com/attendance/listRecord", null, param, body).getRecordresult();
    }

    /**
     * 获取打卡详情
     *
     * @apiNote https://open.dingtalk.com/document/isvapp/upload-punch-records
     */
    @Override
    public boolean uploadAttendanceRecord(String access_token, String userId, String deviceName, String deviceId, String photoUrl, long userCheckTime) {
        Map param = UtilMap.map("access_token", access_token);
        Map body = UtilMap.map("userid, device_name, device_id, photo_url, user_check_time", userId, deviceName, deviceId, photoUrl, userCheckTime);
        return DDR.doPost("https://oapi.dingtalk.com/topapi/attendance/record/upload", null, param, body).isSuccess();
    }

    /**
     * 查询假期余额 [分页是根据游标控制]
     *
     * @apiNote https://open.dingtalk.com/document/isvapp-server/query-holiday-balance
     */
    @Override
    public List<Map> queryVacationQuota(String access_token, String op_userid, String leave_code, String userids, int offset, int size) {
        return _queryVacationQuota(access_token, op_userid, leave_code, userids, offset + size, size, null);
    }

    /**
     * 查询假期余额_全部
     */
    @Override
    public List<Map> queryVacationQuota_all(String access_token, String op_userid, String leave_code, String userids, int offset, int size) {
        return _queryVacationQuota(access_token, op_userid, leave_code, userids, offset + size, size, new ArrayList<>());
    }

    // 递归查询 | 单次查询
    private List<Map> _queryVacationQuota(String access_token, String op_userid, String leave_code, String userids, int offset, int size, List<Map> records) {
        Map body = new HashMap();
        body.put("leave_code", leave_code);
        body.put("op_userid", op_userid);
        body.put("userids", userids);
        body.put("offset", offset);
        body.put("size", size);
        DDR ddr = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/attendance/vacation/quota/list", null, DDConf.initTokenParams(access_token), body, DDR.class);
        List<Map> list = (List<Map>) ((Map) ddr.getResult()).get("leave_quotas");
        if (ObjectUtil.isNull(records)) {
            return list;
        }
        records.addAll(list);
        if (Boolean.TRUE.equals(((Map<?, ?>) ddr.getResult()).get("has_more"))) {
            _queryVacationQuota(access_token, op_userid, leave_code, userids, offset + size, size, records);
        }
        return records;
    }
}
