package com.malk.service.h3yun.impl;

import com.malk.server.common.FilePath;
import com.malk.server.common.VenR;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.h3yun.CYR;
import com.malk.service.h3yun.CYClient;
import com.malk.utils.UtilFile;
import com.malk.utils.UtilHttp;
import com.malk.utils.UtilMap;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.Map;

@Slf4j
@Service
public class CYImplClient implements CYClient {

    /// 请求路径
    private String getRequestUrl(String path) {
        return "https://api.dingtalk.com/v1.0/h3yun" + path;
    }

    /**
     * 获取附件临时免登地址
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/obtain-the-temporary-attachment-free-address
     */
    @Override
    public String getTemporaryUrls(String accessToken, String attachmentId) {
        Map param = UtilMap.map("attachmentId", attachmentId);
        CYR cyr = (CYR) CYR.doGet(getRequestUrl("/attachments/temporaryUrls"), DDConf.initTokenHeader(accessToken), param, VenR.RC_CYR);
        return UtilMap.getString(((Map) cyr.getData()), "attachmentUrl");
    }

    @Autowired
    private FilePath filePath;

    @Override
    public File saveFromTemporaryUrl(String accessToken, Map attachment) {
        String fileUrl = getTemporaryUrls(accessToken, UtilMap.getString(attachment, "FileId"));
        String fileName = UtilMap.getString(attachment, "FileName");
        File file = UtilFile.mkdirIfNot(fileName, filePath.getPath().getFile());
        UtilHttp.doDownload(fileUrl, file);
        return file;
    }
}
