package com.malk.utils;

import cn.hutool.core.util.ObjectUtil;
import com.malk.server.common.McException;
import org.apache.commons.lang3.StringUtils;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

public abstract class UtilMap {

    /************* 快速创建map ************/

    /**
     * 快速创建map [key, Objects]
     */
    public static Map<String, Object> map(String keys, Object... values) {
        String[] props = keys.split(", ");
        if (props.length != values.length) {
            McException.assertParamException_Null(keys);
        }
        Map<String, Object> map = new HashMap<>();
        Arrays.stream(values).forEach(UtilMc.consumerWithIndex((item, index) -> {
            map.put(props[index], item);
        }));
        return map;
    }

    /**
     * 快速创建map [key, Strings]
     */
    public static Map<String, String> map(String keys, String values) {
        String[] props = keys.split(", ");
        String[] contents = values.split(", ");
        if (props.length != contents.length) {
            McException.assertParamException_Null(keys);
        }
        Map<String, String> map = new HashMap<>();
        Arrays.stream(contents).forEach(UtilMc.consumerWithIndex((item, index) -> {
            map.put(props[index], String.valueOf(item));
        }));
        return map;
    }

    /**
     * 快速创建map [保留skeys, 取值ckeys]
     */
    public static Map<String, Object> map(String skeys, String ckeys, Map map) {
        String[] sprops = skeys.split(", ");
        String[] cprops = ckeys.split(", ");
        if (sprops.length != cprops.length) {
            McException.assertParamException_Null(skeys);
        }
        Map data = new HashMap();
        Arrays.stream(cprops).forEach(UtilMc.consumerWithIndex((item, index) -> {
            data.put(sprops[index], map.get(item));
        }));
        return data;
    }

    /**
     * 快速创建map [合并skeys, 取值ckeys]
     */
    public static void map(String skeys, String ckeys, Map sMap, Map cMap) {
        sMap.putAll(map(skeys, ckeys, cMap));
    }

    /************* 赋值 ************/

    /**
     * 赋值 [为空对象, 字符串空null, 忽略]
     */
    public static Map put(Map data, String key, Object value) {
        if (ObjectUtil.isNull(data)) {
            data = new HashMap();
        }
        if (ObjectUtil.isNotNull(value)) {
            data.put(key, value);
        }
        return data;
    }

    /************* 赋值 ************/

    /**
     * 取值 [String.valueOf 避免空指针]
     */
    public static String getString(Map data, String key) {
        if (data.containsKey(key)) {
            return String.valueOf(data.get(key));
        }
        return "";
    }

    /**
     * 取值 [转为 long]
     */
    public static long getLong(Map data, String key) {
        McException.assertParamException_Null(data, key);
        return Long.valueOf(getString(data, key));
    }

    /**
     * 取值 [转为 int]
     */
    public static int getInt(Map data, String key) {
        return Integer.valueOf(getString(data, key));
    }

    /**
     * 取值 [为空返回默认值]
     */
    public static Object get(Map data, String key, Object defaultValue) {
        Object value = data.get(key);
        if (ObjectUtil.isNull(value)) {
            return defaultValue;
        }
        return value;
    }

    /**
     * 取值 [为空返回默认字符串]
     */
    public static String getString(Map data, String key, String defaultValue) {
        String value = getString(data, key);
        if (StringUtils.isBlank(value)) {
            return defaultValue;
        }
        return value;
    }

    /************* 判定 ************/

    /**
     * 是否空字符串
     */
    public static boolean isBlankString(Map data, String key) {
        return StringUtils.isBlank(getString(data, key));
    }

    /**
     * 是否不为空字符串
     */
    public static boolean isNotBlankString(Map data, String key) {
        return StringUtils.isNotBlank(getString(data, key));
    }
}
