package com.malk.utils;

import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Locale;

/**
 * 数字格式化
 */
public class UtilNumber {

    // 货币格式化
    public static final String formatCurrency(Number number) {
        DecimalFormat curFormat = (DecimalFormat) NumberFormat.getCurrencyInstance(Locale.CHINA);
        return curFormat.format(number);
    }

    // 去除CHY货币标识
    public static final String replaceCurrencyCHY(String cur) {
        return cur.replace("￥", "").replace("¥", "");
    }

    // 百分比格式化
    public static final String formatPercent(Number number) {
        DecimalFormat percentFormat = (DecimalFormat) NumberFormat.getPercentInstance(Locale.CHINA);
        percentFormat.setMinimumFractionDigits(2);
        return percentFormat.format(number);
    }

    // 小数位精度格式
    public static final String formatPrecision(double number) {
        DecimalFormat df = new DecimalFormat("#.00");
        return df.format(new BigDecimal(number));
    }

    // 非数值型字符串, 空字符串兼容
    public static final BigDecimal setBigDecimal(String strNum) {
        BigDecimal decimal = null;
        if (StringUtils.isNotBlank(strNum)) {
            try {
                decimal = new BigDecimal(strNum);
            } catch (NumberFormatException e) {
                decimal = BigDecimal.ZERO;
            }
        }
        return decimal;
    }

    // 判断两个字符串, 转数值后是否相等
    public static final int compareBigDecimal(String n1, String n2) {
        return setBigDecimal(n1).compareTo(setBigDecimal(n2));
    }

    public static final boolean equalBigDecimal(String n1, String n2) {
        return compareBigDecimal(n1, n2) == 0;
    }
}
